
-- Hi Rob,
--        I noted with interest Mark Honnor's 3d experiments for his game
-- he's planning on writing in Euphoria. But I noticed that his 3D engine
-- runs rather slowly, even on a Pentium-100. So I re-wrote his routines
-- to make use of Euphoria's SEQUENCE[a..b]=i technique, instead of using
-- polygon(). They're now much faster. I don't know his e-mail address,
-- but I thought that you would be able to forward this to him for me.
 
-- Thanks,
--         Michael Bolin
 
-- INCLUDE FILES
without type_check 
 
include graphics.e
include image.e
 
--include machine.e --dont need these yet!
--include mouse.e
--include ratbag.e
 
-- CONSTANTS
 
--doesn't use trig for perspective,
--but will soon use trig for rotation.
 
--constant pi  = 3.141592654
--constant rad = pi/180
constant X   = 1
constant Y   = 2
 
-- VARIABLES
 
object junk
atom count,t,pos,key
sequence map,sky,page,vp
 
sequence image,         -- a work area inside which the actual drawing is done
	 xy_table,      -- a table of X-Y points
	 slice_table    -- a table of lines to fill to draw the polygons
 
image=repeat(repeat(0,320),200)     -- blank
 
---------------------------------
 
count=0
t=time()
pos=1
key=-1
map = (repeat(repeat(2,100),100))
 
-- now we make the green checkerboard inside the map sequence.
-- it's much faster than calculating for every square we draw
for hy = 1 to 100 do
    for hx = 1 to 100 do
	if remainder(hy+hx,2)=0 then
	    map[hy][hx]=15
	end if
    end for
end for
 
for hx = 1 to 100 do
    for hy = 1 to 100 do
	if rand(50)=1 then
	    map[hx][hy]=1
	end if
    end for
end for
page={1,0}
vp={159,99}
map[20][20]=1
map[21][20]=1
map[22][20]=1
map[23][20]=1
map[24][20]=1
map[20][21]=1
map[21][21]=1
map[22][21]=1
map[23][21]=1
map[21][22]=1
map[22][22]=1
for hx = 2 to 99 do
for hy = 2 to 99 do
    if map[hx][hy]=1 then
	if map[hx][hy+1] !=1 then
	    map[hx][hy+1]=0
	end if
	if map[hx][hy-1] !=1 then
	    map[hx][hy-1]=0
	end if
	if map[hx-1][hy] !=1 then
	    map[hx-1][hy]=0
	end if
	if map[hx+1][hy] !=1 then
	    map[hx+1][hy]=0
	end if
    end if
end for
end for
 
-- FUNCTIONS
 
procedure pause()
    while get_key()=-1 do
    end while
end procedure
 
function pers(atom pointx , atom pointy)
    sequence point2
 
    point2={pointx,pointy}
    point2[X]=point2[X]-vp[X]
    point2[X]=(point2[X]*2)/((point2[Y]*2)/vp[X]+1)
    point2[X]=point2[X]+vp[X]
    point2[Y]=point2[Y]/(point2[Y]/vp[Y]+1)
    return {point2[X],199-point2[Y]}
end function
 
function scan(integer top_y,integer bottom_y)
    -- to create the fill table, we draw each polygon one by one,
    -- then scan the screen to find out where to draw it inside
    -- the image sequence
    sequence pixels,slice_set
    integer left_x,right_x
 
    slice_set={}
    for y_pos=top_y to bottom_y do
	-- using peek() is faster than get_pixel(),
	-- but remember that it can only be used in
	-- graphics mode 19. (which this new version now uses)
	pixels=peek({#A0000+y_pos*320,320})
	left_x=find(15,pixels)  -- find the left side
	if left_x then  -- is there any on this row?
	    -- yes, find the right side
	    right_x=find(0,pixels[left_x..320])
	    if right_x=0 then   -- off the right edge?
		-- yes, stop there
		right_x=320
	    else
		-- add the left side offset
		right_x=right_x+left_x-1
	    end if
	    -- add it to the table
	    slice_set=slice_set & {y_pos+1,left_x,right_x}
	end if
    end for
    return slice_set
end function
 
procedure make_table()
    sequence points,slice_set
 
    xy_table={}
    slice_table={}
    polygon(0,1,{{0,0},{319,0},{319,199},{0,199}})
    for x = -15 to 35 do
	for y = 20 to 1 by -1 do
	    if x+y >5 and x-y <16 then 
		-- calculate the points
		points={
		    pers(x*16-16,y*16),
		    pers(x*16,y*16) , 
		    pers(x*16,y*16-16),
		    pers(x*16-16,y*16-16)}
		-- add the X and Y values to xy_table
		xy_table=append(xy_table,{x,y})
		-- draw the polygon
		polygon(15,1,points)
		-- scan the screen to find the position and shape
		slice_set=scan(floor(points[1][2]-1),floor(points[3][2]+1))
		slice_table=append(slice_table,slice_set)
		-- erase the polygon, so we won't confuse it with the next one
		polygon(0,1,points)
	    end if
	end for
    end for
    polygon(1,1,{{0,0},{319,0},{319,199},{0,199}})
end procedure
 
procedure screen()
    sequence draw_pos,slice
    integer color,x,y,address
 
    for num=1 to length(slice_table) do
	x=xy_table[num][1]
	y=xy_table[num][2]
	if map[x+16][y+pos] !=1 then
	    color=map[x+16][y+pos]
	    slice=slice_table[num]
	    -- now that we have the table, all we do is fill inside the work
	    -- sequence, using Euphoria's sequence-slice fill.
	    -- MUCH faster than polygon()!
	    for s=1 to length(slice) by 3 do
		image[slice[s]][slice[s+1]..slice[s+2]]=color
	    end for
	end if
    end for
    -- it's only necessary to redraw the portion of the screen that
    -- changes
    -- poke() is about 40% faster than pixel(),
    -- but again, it only works in mode 19
    address=#A0000+123*320
    for yp=124 to 200 do
	poke(address,image[yp])
	address=address+320
    end for
end procedure
 
-- SETUP
 
junk = graphics_mode(19)
-- since the actual drawing is now inside a sequence,
-- we don't need to use page flipping.
-- in graphics mode 19, you can use 256 colors, instead of only 16
 
junk = palette(2,{0,35,0})
junk = palette(15,{0,25,0})
junk = palette(14,{0,0,40})
junk = palette(1,{0,0,55})
for w = 0 to 13 do
    junk=palette(13-w,{0,w*2,w*6})
end for
sky=repeat(repeat(0,320),200)
junk = palette(0,{40,30,0})
for w = 1 to 200 do
    sky[w]=repeat(floor(w/20)+3,320)
end for
 
-- PROGRAM
 
------------------------------------
make_table()        -- set up for fast drawing
------------------------------------
 
display_image({0,0},sky)    -- show the sky
 
-- remove the top portion of the sky,
-- the part that doesn't change.
-- this will increase the speed of the main loop slightly,
-- since every screen redraw, we need to set the image sequence
-- back to the sky colors.
sky=sky[124..200]
 
t=time()
 
while key != 27 do
    key = get_key()
    pos = pos+1
    -- remove the old image, and replace it with part of the sky
    -- this is why we removed the top part of the sky sequence,
    -- so we wouldn't have to slice it, too.
    image[124..200]=sky
    screen()
    if pos=80 then
	pos=1
    end if
    count=count+1
end while
t=time()-t
junk=graphics_mode(-1)
printf(1,"%.2f",count/t)
puts(1," Frames Per Second\nHit A Key!")
pause()
 
-- original speed = 1.34 FPS    (486DX2-66)
-- new speed..... = 17.6 FPS    (486DX2-66)
 
-- the routine is now about 13 times as fast!
 
 
 
